/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent, 
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * For contact information, please visit: 
 *      http://www.cs.wisc.edu/condor/nest/
 *
 ***********************************************************************/

#ifndef NEST_CLIENT_POOL_H
#define NEST_CLIENT_POOL_H

#include "nest_common.h"
#include "nest_request.h"
#include "nest_client.h"
#include "nest_pthread_util.h"
#include "nest_protocol.h"
#include "nest_protocol_manager.h"
#include <list>

#define CLIENT_ITERATOR list< nestClient * >::iterator

typedef int nestConnection;


class
nestClientPool {

public:

    nestClientPool( bool thread_safe = false );

    ~nestClientPool( );
    
    int addClient( nestProtocol* protocol, sockaddr_in *address );

    int addClient( nestClient *client );
    
    int removeClient( nestClient *client );

    bool contains( nestClient *client );
    
    nestClient *getFront( bool block = false );
    
    CLIENT_ITERATOR begin( );

    CLIENT_ITERATOR erase( CLIENT_ITERATOR itr );
    
    CLIENT_ITERATOR end( );
    
    int getMaxfd();

    bool isThreadSafe();

    float getAveLen() { return (float)total_sizes / admissions; }

    int getMaxLen() { return max_len; }

    long getAdmissions() { return admissions; }

    bool empty();
    
    ssize_t size( );

    ssize_t clear( );

private:
    bool signal();
    bool wait();
    bool lockMutex( const char *func, int line );
    bool unlockMutex( const char *func, int line );
    
    list <nestClient*> clients;
    long total_sizes;
    int max_len;
    long admissions;
    int maxfd;
    bool thread_safe;
    pthread_mutex_t *mutex;
    pthread_cond_t  *cond_var;
};

#endif

