/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent, 
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * For contact information, please visit: 
 *      http://www.cs.wisc.edu/condor/nest/
 *
 ***********************************************************************/

#ifndef NEST_CONFIG_H
#define NEST_CONFIG_H

/*
===== Adding new configuration variables
If you want to add a new config variable, you need to:

1) Edit the "enum NestConfigVariable" in nest_config.h to list your new
variable.

2) Edit "config_variables" in nest_config.C to match your new enum with a
string.

3) Use GetConfigValue(NestConfigVariable, const char **value) to read your
config variable. A couple of things to note:

    * This returns a bool to tell you if it was defined in the configuration
      file. If it wasn't, don't rely on the char ** being filled in.
    * If you don't care about the value of the variable, pass NULL.

  For examples, see nest_storage_manger.C and nest_protocol_manager.C

4) Update the sample_nest_config with comments to explain your new
   configuration.

===== Debugging the config itself
For most debugging, you can just use the debug config variable.
However, this is only set after reading the config is done.
To debug the config itself, setenv NEST_DEBUG_CONFIG.
*/


enum NestConfigVariable
{
    Config_Debug,
    Config_ConfigFile,
    Config_StorageDir,
	Config_ProxyHost,
    Config_MaintenanceInterval,
    Config_DisableLots,
    Config_MaxLotDuration,
    Config_DefaultLotDuration,
    Config_StorageSpace,
    Config_UserList,
    Config_DenyList,
    Config_AdminUser,
    Config_SystemUser,
    Config_AnonymousUser,
    Config_ProcessUser,
    Config_PasswordFile,
    Config_GroupFile,
    Config_X509CertDir,
    Config_X509HostKey,
    Config_X509HostCert,
    Config_GridMapFile,
    Config_MetaDir,
    Config_ActualGroup,
    Config_NestPort,
    Config_ProxyPort,
    Config_HTTPPort,
    Config_FTPPort,
    Config_GFTPPort,
    Config_NFSPort,
    Config_DebugStderr,
    Config_CondorAdvertise,
    Config_GlobusLocation,
    Config_LogFile,
    Config_DisableKernelQuota,
    Config_FdCacheSize, 
    Config_Daemonize,
    Config_FakeCondorArg,
    Config_DisableAuthentication,
    Config_Latency,
    Config_AdmissionControl,
    Config_Bandwidth,
    Config_ThreadPoolSize,
    Config_PreferredProtocol,
    Config_SchedulingPolicy,
    Config_CacheModelPolicy,
    Config_ConcurrencySelectionPolicy,
    Config_EnableProcessModel,
    Config_EnableEventModel,
    Config_EnableThreadModel,
    Config_ProxysStandAlone,
    Config_FileCreateMode,
    Config_DumpConfigFile,
    Config_PrintHelp,
    NUMBER_OF_CONFIG_VARIABLES
};

/** Print out a sample config file with default values
 */
void DumpSampleConfigFile();


/** Print out the config help
  * If no arg supplied, then print brief usage for all
  * if arg supplied, print detailed usage for that arg
 */
void PrintHelp( const char *command );

/**
 * Reads the configuration file. If the filename is NULL, it 
 * looks for the configuration file. We return true if we
 * read the file okay, false otherwise.
 */
bool ReadConfigFile(int argc, char **argv);

/** Gets a configuration value for a given configuration variable.
 *  Returns true if the variable is defined, false otherwise.
 *  We return a char * in the value--don't edit it or release
 *  it's memory.
 */
bool GetConfigValue(NestConfigVariable variable,
                    const char ** value);

/** Print out the value of all configuration variables. */
void DumpConfig(void);

#endif /* NEST_CONFIG_H */
