/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent, 
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * For contact information, please visit: 
 *      http://www.cs.wisc.edu/condor/nest/
 *
 ***********************************************************************/

#ifndef    _NEST_DEVICE_MANAGER_H_

#define _NEST_DEVICE_MANAGER_H_

#include "nest_arch_specific.h"

#include "nest_common.h"

#define    DE_HASH_SIZE    10

#ifndef NEST_NAMELEN
#define NEST_NAMELEN            40
#endif


typedef struct DiskEntry {
    char            dev[NEST_NAMELEN];
    double          capacity;
    double          user_usage;
    double          group_usage;
} DiskEntry;



class DeviceManager {

    private:
            ClassAdCollection       *devices;

            ViewName                devices_view;
            ClassAdParser           parser;
            ClassAdUnParser         unparser;
            LocalCollectionQuery    query;

            DiskEntry               device_stat;

            int add_entry(DiskEntry *d);
            DiskEntry * get_entry(const char *dev);

    public:
            DeviceManager();
            ~DeviceManager();

            double free_space(const char *dev,
                              int is_user);

            double free_space(int is_user);

            NestReplyStatus get_best_device(char *dev,
                                            double *avail,
                                            int is_user);

            int incr_usage(const char *dev,
                           double size,
                           int is_user);

            int decr_usage(const char *dev,
                           double size,
                           int is_user);

            NestReplyStatus initialize(ClassAdCollection *resources,
                                       char *dev,
                                       double *space,
                                       double usage);

            NestReplyStatus recover(ClassAdCollection *resources,
                                    char *dev);

            NestReplyStatus synchronize();
            NestReplyStatus shutdown();

};


#endif
