/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent, 
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * For contact information, please visit: 
 *      http://www.cs.wisc.edu/condor/nest/
 *
 ***********************************************************************/

#include "nest_hash_table.h"

#include <stdlib.h>
#include <string.h>
#include <map>

struct ltstr
{
    bool operator()(const char* s1, const char* s2) const
    {
        return strcmp(s1, s2) < 0;
    }
};

struct nest_hash_table {
    map< string, void *> *hash;
};

struct nest_hash_table * 
nest_hash_table_create( int bucket_count, int (*hash_func)( const char * ) )
{

    struct nest_hash_table *h;
    h = (struct nest_hash_table*) NestMalloc(sizeof(struct nest_hash_table));
    if(!h) return 0;

    h->hash = new map< string, void *>;
    h->hash->clear();

    return h;
}

void nest_hash_table_delete( struct nest_hash_table *h )
{
    delete( h->hash );
    NestFree(h);
}


void * nest_hash_table_lookup( struct nest_hash_table *h, const char *key )
{
    map<string, void *>::iterator entry = h->hash->find(key);
    if ( entry == h->hash->end() ) {
        return NULL;
    } else {
        return (*entry).second;
    }
}

int 
nest_hash_table_insert( struct nest_hash_table *h, const char *key, 
        void *value )
{
    (*h->hash)[key] = value;
    return 1;
}

int nest_hash_table_remove( struct nest_hash_table *h, const char *key )
{
    map<string, void *>::iterator entry = h->hash->find(key);
    if ( entry != h->hash->end() ) {
        h->hash->erase( entry );
        return 0;
    } else {
        return -1;
    }
}

int
nest_hash_table_size ( struct nest_hash_table *h ) {
    return h->hash->size();
}

int 
nest_hash_table_iterate( struct nest_hash_table *h, 
        int (*iterFunc) ( const char *key, void *item, void *arg ), void *arg )
{
    map<string, void *>::iterator i, j;
    j = h->hash->begin();
    int items = 0;
    while( j != h->hash->end() ) {
        i = j++;
        if( ! iterFunc( ((string)((*i).first)).c_str(), (*i).second, arg ) ){
            return items;
        }
        items++;
    }
    return items;
}

// anyone who uses this better hope their key is at least size 4
// this is now ignored
int nest_hash_int( char *s ) {
    return abs((int)*s);
}

// this is now ignored
int nest_hash_string( char *s )
{
    int r=0;

    for( r=0; *s; s++ ) {
        r = r+*s;
    }

    return r;
}

