/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent, 
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * For contact information, please visit: 
 *      http://www.cs.wisc.edu/condor/nest/
 *
 ***********************************************************************/

#ifndef _NEST_PROTOCOL_H
#define _NEST_PROTOCOL_H

#include "nest_storage_common.h"
#include "dynamic-lard.h"

// forward declaration
class nestRequest;


enum {
       NEST_PROTOCOL,
       HTTP_PROTOCOL,
        FTP_PROTOCOL,
       GFTP_PROTOCOL,
        NFS_PROTOCOL,
      PROXY_PROTOCOL,
    STORAGE_PROTOCOL,
  ADVERTISE_PROTOCOL,
};
    // TODO: is this kludgey or elegant?
    // don't count DISK_PROTOCOL since we don't actually listen for it
#define PROTOCOL_COUNT 6

class nestAd;
class nestClient;

class nestProtocol {

public:

        // I wish the destructor could be pure virtual
    virtual ~nestProtocol() = 0;

    virtual bool sendErrorMessage( const char *filename, const char *func_name,  int error ) = 0;

    virtual bool sendAck(  ) = 0;

    virtual bool sendFilesize( long size) = 0;

    virtual int  getType( ) = 0;

    virtual bool hasSocket() = 0;

    virtual int  getSocket() = 0;

        // MUST NOT BLOCK !!!!  This is exec'ed by main server thread.
    virtual nestRequest* receiveRequest( fd_set *ready ) = 0;

    virtual bool sendDirectoryList(  NestFileInfo *list );

    virtual NestReplyStatus sendProxyIndex( DynamicLard *index );
    
    virtual bool sendNestLot( NestLot *rstat);

    virtual bool sendLotHandle( NestLotHandle *handle);

    virtual bool sendAd( nestAd *ad );

    virtual bool sendPwd( const char *pwd );

    virtual bool sendVersion( float version );

    virtual bool sendDiskspace( double avail, double total );

    virtual bool sendFinalAck( int transferType );

    virtual bool sendInitialAck(  );

    virtual bool sendTransferInfo( class nestRequest *req );

    virtual bool sendTimestamp(  );

    virtual bool sendBandwidthStatistics(  );

    virtual NestReplyStatus sendFileinfo( NestFileInfo *info );

    virtual NestReplyStatus setFileattr( NestFileInfo *info, const char *fname, void *fattr );
    
    virtual bool sendPhysicalName( const char *physical_name );

    virtual bool sendAcl( const char *user,
                          int is_user,
                          nest_access_rights_t access );
    virtual bool sendAcl( const NestACLList *Acl );

    virtual bool sendProxyList( list <char *> *proxy ); 

    virtual bool persistent();

        // we used to use this in concurrency_nob but I think it may not
        // actually be necessary so I've disabled it.  if it makes problems
        // then re-enable
    virtual nestProtocol *clone();

    virtual NestReplyStatus receive_state( int pipe );

    virtual NestReplyStatus send_state( int pipe );


    virtual NestReplyStatus sendFilehandle( NestFileInfo *fi );

        // MUST NOT BLOCK !!!!  This is exec'ed by main server thread.
    virtual NestReplyStatus gssAccept( char **username, char **globus_username );
    
    virtual ssize_t Receive( char *buffer, ssize_t n )= 0;

    virtual ssize_t Receiven( char *buffer, ssize_t n );

    virtual ssize_t Send( const string &buffer );

    virtual ssize_t Send( const char *buffer, ssize_t n ) = 0;

    virtual ssize_t Sendn( const char *buffer, ssize_t n );

    virtual bool endThirdPartyTransfer( int type );

};


#define PROTOCOL_WARN nest_debug( NEST_DEBUG_PROTOCOL, \
        "nest_protocol for type %d: default method used %s: %d\n", \
        getType(), \
        __FILE__, __LINE__ );

#define PROTOCOL_DIE nest_die( \
        "nest_protocol for type %d: default method used %s: %d\n", \
        getType(), \
        __FILE__, __LINE__ );

#include "nest_common.h"

#endif
