/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent, 
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * For contact information, please visit: 
 *      http://www.cs.wisc.edu/condor/nest/
 *
 ***********************************************************************/

#ifndef _NEST_PROTOCOL_NEST_H
#define _NEST_PROTOCOL_NEST_H

#include "nest_common.h"
#include "nest_protocol.h"
#include "nest_speak.h"
#include "nest_speak.i.h"
#include "nest_client.h"


class nestProtocolNest : public nestProtocol {
public:
    static int listen( int port );

    static nestProtocol* accept( int socket, sockaddr_in *address );

    static NestReplyStatus initiateThirdParty( nestClient *client,
            nestProtocol **connection );

    nestProtocolNest( int socket );

    virtual ~nestProtocolNest();

    virtual NestReplyStatus gssAccept( char **username, char **globus_username );
    
    virtual nestRequest* receiveRequest( fd_set *ready );

    virtual bool endThirdPartyTransfer( int type );

    virtual bool sendVersion( float version );
    
    virtual bool sendPwd( const char *pwd );

    virtual bool sendDirectoryList( NestFileInfo *list );

    virtual bool sendAd( nestAd *ad );

    virtual bool sendDiskspace( double avail, double total  );

    virtual bool sendErrorMessage(  const char* filename, const char* func_name,  int error );

    virtual bool sendAck( );

    virtual bool sendFinalAck( int transferType );

    virtual bool sendTransferInfo( nestRequest *req );

    virtual bool sendTimestamp( );

    virtual bool sendFilesize( long size);

    virtual bool sendPhysicalName( const char *name );

    virtual bool sendAcl( const char *user,
                          int is_user,
                          nest_access_rights_t access );

    virtual bool sendAcl( const NestACLList *Acl);

    virtual bool sendNestLot( NestLot *rstat );

    virtual bool sendLotHandle( NestLotHandle *handle );

    virtual bool sendProxyList( list <char *> *proxys );
    
    virtual ssize_t Receive( char *buffer, ssize_t n );

    virtual ssize_t Send( const char *buffer, ssize_t n );

    virtual int getType( ) { return NEST_PROTOCOL; }

    virtual bool hasSocket() { return true; }

    virtual nestProtocol *clone();

    int getSocket() { return socket; }

protected:
    //static sockaddr_in cliaddr;
    //static unsigned addr_sz;

    bool sendMessage( const void *vptr );
    nestRequest* parseRequest( char *msg );
    bool sendTransferInfo( int csd, char *file, int type, long len );
    int socket;
    char msg[NEST_MESSAGESIZE];
    int  msgoffset;

};

#endif
