#include "nest_common.h"
#include "nest_proxy_fetch.h"
#include "nest_client.h"


nestProxyFetch::nestProxyFetch( const char *homefile,
                                const char *proxyfile,
                                const char *shadowfile,
                                long  home_offset,
                                long  home_length,
                                nestClient *client,
                                bool forwarded_request )
{
    assert( homefile   != NULL );
    assert( proxyfile  != NULL );
    assert( shadowfile != NULL );
    assert( client != NULL );

    this->homefile   = NestStrdup( homefile );
    this->proxyfile  = NestStrdup( proxyfile );
    this->shadowfile = NestStrdup( shadowfile ); 

	this->client = client;

    this->home_off = home_offset;
    this->home_len = home_length;

    this->server     = NULL;
    this->attempts   = 0;
    this->forwarded  = forwarded_request;
}

nestProxyFetch::~nestProxyFetch() {
    if ( homefile )     NestFree( homefile );
    if ( proxyfile )    NestFree( proxyfile );
    if ( shadowfile )   NestFree( shadowfile );
    if ( server )       NestFree( server );
}

void
nestProxyFetch::setServer( const char *server ) {
    if ( this->server != NULL ) NestFree( this->server );
    this->server = NestStrdup( server );
}

bool
nestProxyFetch::wasForwarded() {
    return forwarded;
}

const char *
nestProxyFetch::getServer() {
    return server;
}

const char *
nestProxyFetch::getHomeFile() {
    return homefile;
}

const char *
nestProxyFetch::getProxyFile() {
    return proxyfile;
}

user_id_t
nestProxyFetch::getClientUid() {
    return client->getUid();
}

const char *
nestProxyFetch::getShadowFile() {
    return shadowfile;
}

const char *
nestProxyFetch::getClientName() {
    return client->getName();
}

const char *
nestProxyFetch::getClientIdentity() {
    return client->getIdentity();
}

const char *
nestProxyFetch::getClientCwd() {
    return client->getCwd();
}

const nestClient *
nestProxyFetch::getClient() {
    return client;
}

long
nestProxyFetch::getHomeOffset() {
    return home_off;
}

long
nestProxyFetch::getHomeLength() { 
    return home_len;
}

int
nestProxyFetch::getAttempts() {
    return attempts;
}

void
nestProxyFetch::incrementAttempts() {
    attempts++;
}
