/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent, 
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * For contact information, please visit: 
 *      http://www.cs.wisc.edu/condor/home/
 *
 ***********************************************************************/

#ifndef _NEST_PROXY_FETCH_H
#define _NEST_PROXY_FETCH_H

#include "nest_common.h"

class nestClient;

class
nestProxyFetch {

public:

    nestProxyFetch(     const char *homefile,     // file on home node
                        const char *proxyfile,    // file on proxy node
                        const char *shadowname,   // shadow name on proxy node
                        long  homefile_offset,
                        long  homefile_length,
						nestClient *client,
                        bool forwarded_req = false);   
                                                // is client themselves a proxy

    ~nestProxyFetch( );

    const char* getServer( );
    const char* getHomeFile();
    const char* getProxyFile();

	const nestClient *getClient( );
    const char *getClientName();
    const char *getClientIdentity();
    const char *getClientCwd();
    user_id_t   getClientUid();

    const char *getShadowFile();

    long getHomeOffset();
    long getHomeLength();

    int  getAttempts();
    bool wasForwarded();

    void setStatus( NestReplyStatus status );
    void incrementAttempts();
    void setServer( const char *server );

private:
    int attempts;

    char *homefile;
    char *proxyfile;
    char *shadowfile;
    char *server;
	nestClient *client;

    long home_off;
    long home_len;

    bool forwarded;
};

#endif
