/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent, 
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * For contact information, please visit: 
 *      http://www.cs.wisc.edu/condor/nest/
 *
 ***********************************************************************/

#ifndef    _NEST_QUOTA_MANAGER_H_

#define _NEST_QUOTA_MANAGER_H_

#include <sys/stat.h>
#include <sys/time.h>
#include <stdlib.h>
#include <string.h>

#include "nest_common.h"
#include "nest_protocol.h"

#include "nest_quota_instance.h"

#define  NUM_QUOTA_INSTANCES  1

#include "fs_ext2_quota_instance.h"
#define  FS_EXT2              0

typedef struct qimap {
    char            dev[NEST_NAMELEN];
    int             qi_index;
    struct qimap    *next;
} QuotaInstanceMap;


class QuotaManager {

    private:

        ClassAdCollection       *quota_catalog;

        ViewName                quota_catalog_view;
        ClassAdParser           parser;
        ClassAdUnParser         unparser;
        LocalCollectionQuery    query;

        /*
            QuotaInstanceMap - maps a device to the respective
                                QuotaInstance or Quota-Enforcement.

            quota_inst_map is implemented a linked-list and
            locate_quota_inst() is used to search thru it.
            eg:{
                 {"/dev/hda5", FS_EXT2},
                 {"/dev/sdb6", FS_EXT2},
                 {"/mem",      MEM}, ...
               }
        */

        QuotaInstanceMap    *quota_inst_map;
        QuotaInstance       *quota_inst[NUM_QUOTA_INSTANCES];

        int locate_quota_inst(const char *dev);

        NestReplyStatus add_entity(const char *name,
                                   int is_user,
                                   const char *dev,
                                   double size,
                                   int inode_count);

        NestReplyStatus delete_entity(const char *name,
                                      int is_user,
                                      const char *dev);

        NestReplyStatus update_entity(const char *name,
                                      int is_user,
                                      const char *dev,
                                      double new_size,
                                      int new_inode_count);

        NestReplyStatus view_quota_status(const char *name,
                                          int is_user,
                                          const char *dev,
                                          double *size,
                                          int *inode_count);

        NestReplyStatus view_quota_usage(const char *name,
                                          int is_user,
                                          const char *dev,
                                          double *size,
                                          int *inode_count);


    public:

        QuotaManager();
        ~QuotaManager();


        NestReplyStatus incr_quota(const char *name,
                                   int is_user,
                                   const char *dev,
                                   double more_size);

        NestReplyStatus decr_quota(const char *name,
                                   int is_user,
                                   const char *dev,
                                   double less_size);

        NestReplyStatus initialize(ClassAdCollection *resources,
                                   const char *dev,
                                   double *user_usage);

        NestReplyStatus recover(ClassAdCollection *resources,
                                const char *dev);

        NestReplyStatus shutdown();
        NestReplyStatus synchronize();

        static NestReplyStatus has_kernel_support(const char *dev);
};



#endif
