/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent, 
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * For contact information, please visit: 
 *      http://www.cs.wisc.edu/condor/nest/
 *
 ***********************************************************************/

#ifndef NEST_SPEAK_H
#define NEST_SPEAK_H

#include "nest_error.h"
#include "nest_common.h"
#include "nest_auth.h"

#define NEST_SERVER_PORT 7878

typedef int NestConnection;

typedef void NestProgressFunction( int sofar,
                                   int total,
                                   void *args );

typedef void NestListDirFunction( NestFileInfo *finfo,
                                  void *args);

typedef void NestProxyListFunction( char *proxy, void *args);

typedef void NestAclFunction( const NestACL *acl, void *CbData );

    // opening and closing connections
NestReplyStatus NestOpenConnection( NestConnection &con,
                                    char *nestlocation,
                                    int port = NEST_SERVER_PORT );

NestReplyStatus NestOpenAnonymously( NestConnection &con,
                                    char *nestlocation,
                                    int port = NEST_SERVER_PORT );  

NestReplyStatus NestCloseConnection( NestConnection con );


    // sending
NestReplyStatus NestSendFile(char *localfile,
                             char *remotefile,
                             NestConnection);

NestReplyStatus NestSendFile(char *localfile,
                             char *remotefile,
                             NestConnection,
                             NestProgressFunction *,
                             void *args );

NestReplyStatus NestWriteBytes ( NestConnection con,
                                 char *remotefile,
                                 long offset,
                                 char *buffer,
                                 int size,
                                 bool ack_wanted );

NestReplyStatus NestThirdPartyPut( char *nestfile,
                                   char *remotehost,
                                   char *remotefile,
                                   char *remoteprotocol,
                                   NestConnection con );

    // receiving
NestReplyStatus NestReceiveFile( char *remotefile,
                                 char *localfile,
                                 NestConnection );

NestReplyStatus NestReceiveFile( char *remotefile,
                                 char *localfile,
                                 NestConnection,
                                 NestProgressFunction *,
                                 void *args );

NestReplyStatus NestReadBytes( NestConnection con,
                               char *remotefile,
                               long offset,
                               char *buf,
                               int size,
                               int &bytes_read );

NestReplyStatus NestThirdPartyGet( char *nestfile,
                                   char *remotehost,
                                   char *remotefile,
                                   char *remoteprotocol,
                                   NestConnection con );

    // collections
NestReplyStatus NestNameCollection( const char *path,
                                const char *name,
                                NestConnection con );

NestReplyStatus NestUnnameCollection( const char *name,
                                  NestConnection con );
NestReplyStatus NestDateCollection( const char *name,
									time_t date,
									NestConnection con );
NestReplyStatus NestVersionCollection( const char *name,
									   const char *version,
									   NestConnection con );
NestReplyStatus NestCommentCollection( const char *name,
									const char *comment,
									NestConnection con );


    // group and user management
NestReplyStatus NestAddUser( NestConnection con,
                             char *user,
                             char *group,
                             char *identity = NULL );

NestReplyStatus NestAddGroup( NestConnection con,
                              char *group);

NestReplyStatus NestDelUser( NestConnection con,
                             char *user);

NestReplyStatus NestDelGroup( NestConnection con,
                              char *group);

NestReplyStatus NestAddUserToGroup( NestConnection con,
                                char *user,
                                char *group);

NestReplyStatus NestDelUserFromGroup( NestConnection con,
                                  char *user,
                                  char *group);

    // lot management
NestReplyStatus NestRequestUserLot( NestLotHandle *handle,
                                    NestConnection con,
                                    double size_in_MB,
                                    double minutes,
                                    char *user = NULL,
                                    char *parent_user = NULL,
                                    char *parent_group = NULL,
                                    NestLotHandle *parent_handle = NULL,
                                    int lot_type = G_LOT);

NestReplyStatus NestRequestGroupLot( NestLotHandle *handle,
                                     NestConnection con,
                                     double size_in_MB,
                                     double minutes,
                                     char *group,
                                     char *parent_user = NULL,
                                     char *parent_group = NULL,
                                     NestLotHandle *parent_handle = NULL,
                                     int lot_type = G_LOT);

NestReplyStatus NestListUserLots( NestLotList **list,
                                  NestConnection con,
                                  const char *user = NULL );

NestReplyStatus NestListGroupLots( NestLotList **list,
                               NestConnection con,
                               char *group );

NestReplyStatus NestGetLotStat( NestLot *result,
                            NestConnection con,
                            NestLotHandle *handle );

NestReplyStatus NestUpdateLot( NestConnection con,
                           NestLotHandle *handle,
                           double more_size_in_MB,
                           double more_minutes);

NestReplyStatus NestChangeLotType( NestConnection con,
                                   NestLotHandle *handle,
                                   int lot_type,
                                   double more_minutes);

NestReplyStatus NestLotMoveFile( NestConnection con,
                                 const char *file,
                                 NestLotHandle *source,
                                 NestLotHandle *dest,
                                 double move_size_in_bytes );

NestReplyStatus NestTerminateLot( NestConnection con,
                              NestLotHandle *handle);

NestReplyStatus NestAddUserToLot( NestConnection con,
                                  NestLotHandle *handle,
                                  const char *user);
   
NestReplyStatus NestDeleteUserFromLot( NestConnection con,
                                       NestLotHandle *handle,
                                       const char *user);

NestReplyStatus NestAttachLotToFile( NestConnection con,
                                     NestLotHandle *handle,
                                     const char *file);

NestReplyStatus NestDetachLotFromFile( NestConnection con,
                                       NestLotHandle *handle,
                                       const char *file);

NestReplyStatus NestChangeUserID( char *username,
                                  NestConnection con,
                                  char *server );

    // proxy stuff
NestReplyStatus NestProxyRegister( char *nestlocation, NestConnection con );

NestReplyStatus NestProxyQuery( NestConnection con,
                                NestProxyListFunction *foreachproxy,
                                void *args );

    // miscellaneous
NestReplyStatus NestFormat( float &bytesavailable,
                            float &totalbytes,
                            NestConnection con );

NestReplyStatus NestSync( NestConnection con );

NestReplyStatus NestAvailableSpace( float &bytesavailable,
                                    float &totalbytes,
                                    NestConnection con);

NestReplyStatus NestDebug( char *debug_levels,
                           NestConnection con );

NestReplyStatus NestListFiles( NestConnection con,
                               NestListDirFunction *foreachfile,
                               void *args );

NestReplyStatus NestFilesize( long &filesize,
                              char *remotefile,
                              NestConnection con );

NestReplyStatus NestRemoveFile( char *remotefile,
                                NestConnection con );

NestReplyStatus NestRenameFile( char *oldfile,
                                char *newfile,
                                NestConnection con );

NestReplyStatus NestGetTimestamp( long &timestamp,
                                  NestConnection con );

NestReplyStatus NestGetPwd( char **buf,
                        NestConnection con );

NestReplyStatus NestGetPhysicalName( char **buf,
                                 const char *file,
                                 NestConnection con );

/*
   this can be easily turned into a "real" classad if the client wants to:
    ClassAdParser cap;
    ClassAd *ad = cap->ParseClassAd( adc );
    ...
    free( ad );
*/
NestReplyStatus NestRequestAd( NestConnection con, char **ad, unsigned *adlen );

NestReplyStatus NestGetAcl( NestConnection con,
                            const char *file,
                            const char *user,
                            NestAclFunction *Callback,
                            void *CbData,
                            bool is_group );
NestReplyStatus NestSetAcl( NestConnection con, 
                            const char *file,
                            const char *user,
                            nest_access_rights_t access,
                            bool is_group );

NestReplyStatus NestGetVersion( float *version,
                            NestConnection con );

NestReplyStatus NestChangeDir( const char *dir,
                           NestConnection con );

NestReplyStatus NestMkDir( const char *dir,
                       NestConnection con );

NestReplyStatus NestRmDir( const char *dir,
                       NestConnection con );

NestReplyStatus NestRmRecursive( const char *dir,
                       NestConnection con );

int NestReplyStatusToErrno( NestReplyStatus error );

NestReplyStatus NestLink( const char *oldPath,
                          const char *newPath,
                          int isSymbolic,
                          NestConnection con );
    
NestReplyStatus NestCopy( const char *oldPath,
                          const char *newPath,
                          NestConnection con);

NestReplyStatus NestRemoteShutdown( NestConnection con );

void NestFormatAccessString( nest_access_rights_t access,
                             char *access_string );

NestReplyStatus NestParseAccessString( nest_access_rights_t *access,
                                       const char *string );
#endif
