/***********************************************************************
 *
 * NeST: Network Storage Technologies
 * Copyright (C) 2001-2002, Computer Sciences Department,
 * University of Wisconsin-Madison, WI, and John Bent,
 * Venkateshwaran Venkataramani, and other members of the Condor Project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * For contact information, please visit:
 *      http://www.cs.wisc.edu/condor/nest/
 *
 ***********************************************************************/

#ifndef _NEST_STORAGE_COMMON_H_

#define _NEST_STORAGE_COMMON_H_

#include <sys/types.h>
#include <sys/time.h>
#include <sys/param.h>

#include "nest_arch_specific.h"

#define IS_USER             1
#define IS_GROUP            0

#define MINUTES(x)          (x)
#define HOURS(x)            ((x) * MINUTES(60))
#define DAYS(x)             ((x) * HOURS(24))
#define WEEKS(x)            ((x) * DAYS(7))
#define MONTHS(x)           ((x) * DAYS(30))
#define YEARS(x)            ((x) * DAYS(365))

#define NEST_BLK_SIZE        1024

#define NUM_BLKS(x)         ((int)(MB_BLKS(x) + 0.999999))
#define    SIZE_IN_MB(x)        ( (double) (((double) (x)) / 1024) )

#define KB_BLKS(x)          (x)
#define MB_BLKS(x)          (KB_BLKS(x) * 1024)
#define GB_BLKS(x)          (MB_BLKS(x) * 1024)

#define DEFAULT_LOT_DURATION    (DAYS(1))
#define INFINITE_LOT_DURATION   (525600000) // 1000 years in minutes!!

//  Ratio between data to number of inodes!
#define DATA_KB_PER_INODE   8    //    in KB, of course

#define NEST_EVERYBODY            "nest_everybody"
#define NEST_NOBODY                "nest_nobody"

#define    NEST_NO_GROUP            "no_nest_group"



typedef long unsigned int    user_id_t;

#define        MAX_USR_ENTRIES    100

struct group_ent {
    char    grp_name[NEST_NAMELEN];
    char    grp_passwd[NEST_NAMELEN];
    gid_t    grp_gid;
    char    grp_entries[MAX_USR_ENTRIES][NEST_NAMELEN];
};

typedef struct group_ent group_ent;

#endif
