#ifndef _NEST_STORAGE_MANAGER_H
#define _NEST_STORAGE_MANAGER_H

#include "nest_arch_specific.h"

#include "nest_common.h"
#include "nest_protocol_storage.h"
#include "nest_client.h"

#include "nest_identity_manager.h"
#include "nest_lot_manager.h"
#include "nest_file_catalog.h"
#include "nest_collection_manager.h"

#include "nest_hash_table.h"

class
nestStorageManager {

public:

    //  Initialize  Storage Manager
    static int  init( );

    static NestReplyStatus  initialize();

    static void writeAd( nestProtocol *);

    static NestReplyStatus  sync();

    static NestReplyStatus  maintenance();
    static NestReplyStatus  shutdown();

    //  Meta-file operations
    static nestProtocol*    openMetaFile(char *file,
                                         int flags );

    static NestReplyStatus  renameMetaFile(char *oldpath,
                                           char *newpath );

    static NestReplyStatus  statMetaFile(char *file,
                                         nest_stat *st );

    static NestReplyStatus  unlinkMetaFile( char *file );

    static char*            metaFilepath(char *file );

    static long             metaFilelength( char *file );

    //  File operations
    static NestReplyStatus  open(nestProtocol **result,
                                 const char *user,
                                 user_id_t  uid,
                                 const char *group,
                                 const char *file,
                                 int flags,
                                 off_t offset,
                                 ssize_t filesize,
                                 const char *cwd,
                                 int link_count = 0,
								 int importance = 1 );

    static char *           filehandleToPath( NestFilehandle * );
    
    static NestReplyStatus  pathToFilehandle(const char *user,
                                            const char *group,
                                            const char *file,
                                            const char *cwd,
                                            NestFilehandle   *inode_no,
                                            int   link_count = 0);



    static NestReplyStatus  close(nestProtocol *connection,
                                  nestClient *client = NULL);

    static NestReplyStatus  deleteLink(const char *filename);

    static NestReplyStatus  deleteDirContents(const char *user,
                                              const char *filename);

    static NestReplyStatus  deleteFile(const char *user,
                                       const char *filename,
                                       const char *cwd,
                                       bool update_lot = true);

    static NestReplyStatus  renameFile( const char *user,
                                        user_id_t  uid,
                                        const char *oldpath,
                                        const char *newpath,
                                        const char *cwd,
                                        bool update_lot = true );

    static NestReplyStatus  get_type( nest_obj_type_t   *result,
                                      const char *filename,
                                      const char *user,
                                      const char *cwd );

    static NestReplyStatus  get_physical_filename(char **physical,
                                                  const char *filename,
                                                  const char *user,
                                                  const char *cwd,
                                                  int link_count = 0);

    static NestReplyStatus  Link(const char *user,
                                 const char *oldpath,
                                 const char *newpath,
                                 int val,
                                 const char *cwd);

    static NestReplyStatus  sync(const char *user);


    //  Directory operations

    static NestReplyStatus frame_full_path( string &result,
                                            const char *user,
                                            const char *file );

    static NestReplyStatus expand_path( string &result,
                                        const char *user,
                                        const char *cwd,
                                        const char *file );


    static NestReplyStatus get_dest_dir( char *dest,
                                         const char *cwd,
                                         const char *dir,
                                         const char *user );

    static NestReplyStatus chDir(char **new_cwd,
                                 const char *user,
                                 const char *cwd,
                                 const char *dir);

    static NestReplyStatus mkDir(const char *user,
                                 const char *cwd,
                                 const char *dir,
                                 user_id_t uid );

    static NestReplyStatus rmDir(const char *user,
                                 const char *cwd,
                                 const char *dir,
                                 bool recursive = false );

    static char*            getRootDir( );

    static NestFileInfo*    directoryContents(const char *user,
                                              const char *dir );


    //  User/Group Management operations
    static NestReplyStatus  add_user(const char *username,
                                     const char *groupname,
                                     const char *cur_user,
                                     const char *identity);

    static NestReplyStatus  add_user(const char *username,
                                     const char *cur_user);

    static NestReplyStatus  delete_user(const char *username,
                                        const char *cur_user);

    static NestReplyStatus  add_group(const char *groupname,
                                      const char *cur_user);
    static NestReplyStatus  delete_group(const char *groupname,
                                         const char *cur_user);

    static NestReplyStatus  add_user_to_group(const char *username,
                                              const char *groupname,
                                              const char *cur_user);
    static NestReplyStatus  delete_user_from_group(const char *username,
                                                   const char *groupname,
                                                   const char *cur_user);

    //  Access control operations
    static NestReplyStatus  get_acl(nest_access_rights_t *result,
                                    const char *for_name,
                                    int is_user,
                                    const char *file,
                                    const char *user,
                                    const char *cwd);

    static NestReplyStatus  get_all_acl(NestACLList **result,
                                        const char *file,
                                        const char *user,
                                        const char *cwd);

    static NestReplyStatus  set_acl(nest_access_rights_t arg,
                                    const char *to_name,
                                    int is_user,
                                    const char *file,
                                    const char *user,
                                    const char *cwd);

    static NestReplyStatus  get_user_acl(nest_access_rights_t *result,
                                         const char *for_user,
                                         const char *file,
                                         const char *user,
                                         const char *cwd);

    static NestReplyStatus  set_user_acl(nest_access_rights_t arg,
                                         const char *to_user,
                                         const char *file,
                                         const char *user,
                                         const char *cwd);

    static NestReplyStatus  get_group_acl(nest_access_rights_t *result,
                                          const char *for_group,
                                          const char *file,
                                          const char *user,
                                          const char *cwd);

    static NestReplyStatus  set_group_acl(nest_access_rights_t arg,
                                          const char *to_group,
                                          const char *file,
                                          const char *user,
                                          const char *cwd);

    //  Lot operations
    static NestReplyStatus request_user_lot(NestLotHandle *handle,
                                            const char *user,
                                            double size,
                                            double duration,
                                            const char *lot_user,
                                            const char *parent_user,
                                            const char *parent_group,
                                            NestLotHandle *parent_handle,
                                            int lot_type=G_LOT,
                                            int cost=1);

    static NestReplyStatus request_group_lot(NestLotHandle *handle,
                                             const char *user,
                                             double size,
                                             double duration,
                                             const char *lot_group,
                                             const char *parent_user,
                                             const char *parent_group,
                                             NestLotHandle *parent_handle,
                                             int lot_type=G_LOT,
                                             int cost=1);

    static NestReplyStatus list_user_lots(NestLotList **result,
                                          const char *user,
                                          const char *lot_user );

    static NestReplyStatus list_group_lots(NestLotList **result,
                                           const char *req_user,
                                           const char *group );

    static NestReplyStatus update_lot(NestLotHandle *handle,
                                      const char *user,
                                      double size,
                                      double duration,
                                      int cost = 0);

    static NestReplyStatus change_lot_type(NestLotHandle *handle,
                                           const char *user,
                                           int lot_type,
                                           double duration);

    static NestReplyStatus modify_lot_usage(const char *user,
                                            const char *filename,
                                            const char *cwd,
                                            NestLotHandle  *from_lot,
                                            double from_share,
                                            NestLotHandle  *to_lot);

    static NestReplyStatus get_lot_status(NestLot* result,
                                          NestLotHandle *handle,
                                          const char *user);

    static NestReplyStatus set_user_default_lot( NestLotHandle *handle,
                                                 const char *user );

    static NestReplyStatus terminate_lot(NestLotHandle *handle,
                                         const char *user);

    static NestReplyStatus add_user_to_lot(NestLotHandle *handle,
                                           const char *cur_user,
                                           const char *user);

    static NestReplyStatus delete_user_from_lot(NestLotHandle *handle,
                                                const char *cur_user,
                                                const char *user);

    static NestReplyStatus attach_lot_to_file(NestLotHandle *handle,
                                              const char *file,  
                                              const char *user,
                                              const char *cwd);

    static NestReplyStatus detach_lot_from_file(NestLotHandle *handle,
                                                const char *file,  
                                                const char *user,
                                                const char *cwd);

    static NestReplyStatus increaseAllocation(nestClient *client);
    static NestReplyStatus resetAllocation(const char *file,
                                           const char *user,
                                           const char *cwd,
                                           long offset); 
    
    static NestReplyStatus cleanupGftpTransfer(nestClient *client);
                                                
    //  Other operations
    static NestReplyStatus  bytesAvailable( int is_user,
                                            long long int *free,
                                            long long int *total );

    static NestReplyStatus  filelength(  long *filesize,  
                                         const char *filename,
                                         const char *user,
                                         const char *cwd );

    static NestReplyStatus  fileusage(  double *size,
                                        const char *file,
                                        const char *user, 
                                        const char *cwd );

    static NestReplyStatus  fileinfo(  NestFileInfo *info,
                                       const char *filename,
                                       const char *user,
                                       const char *cwd,
                                       int link_count = 0 );

    static char *           getInitialClientDir( );
    
    static bool             is_valid_user(const char *user, user_id_t *uid );

    static NestReplyStatus  formatDisk(const char *user,
                                       const char *group );

    static NestReplyStatus  rememberLength( 
            const char *user, const char *cwd,
            const char *file, long *length );

    static  NestFileCatalog         *file_catalog;
    static  NestIdentityManager     *identity_manager;
    static  NestLotManager          *lot_manager;
	static  NestCollectionManager   *collection_manager;

    static char *           type();

    static bool             is_lots_enabled();

private:

    static  double                  max_lot_duration;
    static  double                  default_lot_duration;
    static  double                  storage_space;
    static  char *                  userlist;
    static  char *                  denylist;

    static  mode_t                    file_create_mode;


    static  struct nest_hash_table *     fh_hash;

    //static  ClassAdCollectionServer *Resources;
    static  ClassAdCollection       *Resources;
    static  ClassAdParser           parser;
    static  ClassAdUnParser         unparser;
    static  LocalCollectionQuery    query;

    static int chRootDir( );
    static int chMetaDir( );
    static NestReplyStatus  get_home_dir(string &result,
                                         const char *user);
    static NestReplyStatus  add_home_dir(const char *user,
                                         const char *group);
    static NestReplyStatus  delete_home_dir(const char *user);

    static void cache_uniquifier (  string &logical,
                                    NestFilehandle *uniq,
                                    nest_obj_type_t type );

};

#endif
